; FLAMES.ASM (demo)
; Re-Written by Erdogan Tan (by removing message procedures) in MASM format
; 12-3-2005
; Assembler: Microsoft Macro Assembler 6.11 "MASM.EXE"
;
;=============================================================================
; flame.asm - Plasma-Like Flames Demo.
;
; Copyright (c) 1993, Carlos Hasan                  Last modified: 22/11/93
;
; Description:
;  Plasma Like Flames effect based on Vangelis Fire Code.
;
; Assembler:
; Borland TASM
;
;=============================================================================

;============================ Code Segment ===================================

        .model tiny
        .286

        .code
        .STARTUP

;-----------------------------------------------------------------------------
; Start - Startup code called from DOS.
;-----------------------------------------------------------------------------

        org 100h

proc_start      proc near
        ; Do Flames Demo
        mov     ax,0013h                ; Set VGA Linear 320x200x256
        int     10h
        mov     dx,3C4h                 ; Disable Chain-Four
        mov     ax,0604h
        out     dx,ax
        mov     dx,3C4h                 ; Enable Write to All Four Planes
        mov     ax,0F02h
        out     dx,ax
        mov     ax,0A000h               ; Clear Display Memory
        mov     es,ax
        xor     di,di
        xor     ax,ax
        mov     cx,8000h
        cld
        rep     stosw
        mov     dx,3D4h                 ; Reprogram CRT Controller:
        mov     ax,00014h               ; turn off dword mode
        out     dx,ax
        mov     ax,0e317h               ; turn on byte mode
        out     dx,ax
        mov     ax,00409h               ; cell height
        out     dx,ax

        call    proc_WaitVR
        lea     si,[Palette]            ; Sets Color Palette
        mov     cx,768
        mov     dx,3C8h
        xor     al,al
        out     dx,al
        inc     dx
        cld
        rep     outsb
        mov     ax,ds                   ; Clears Frame Buffer
        mov     es,ax
        lea     di,[Frame]
        mov     cx,XDOTS*(YDOTS+HIDDEN)
        xor     ax,ax
        rep     stosw
        mov     ax,0A000h               ; Set Display Memory Segment
        mov     es,ax
        mov     bx,FLSEED               ; Loads Flames Seed Factor
        xor     bp,bp                   ; Start TIMEOUT Counter
FlamesLoop:
        call    proc_WaitVR             ; Wait Vertical Retrace
        lea     si,[Frame]              ; Write Frame to the Screen
        xor     di,di
        mov     cx,XDOTS*YDOTS/2
WriteFrameLoop:
        lodsw
        mov     dl,al
        lodsw
        mov     dh,al
        mov     ax,dx
        stosw
        loop    WriteFrameLoop

        mov     cx,XDOTS*(YDOTS+HIDDEN-2) ; Transform Upper Frame Lines
        lea     si,[Frame+2*XDOTS]
FlamesTransform:
        mov     ax,[si-2]               ; Sets Pixel at (X,Y) like
        add     ax,[si+0]               ; the average of the near four
        add     ax,[si+2]               ; pixels below it:
        add     ax,[si+2*XDOTS]         ; P(X,Y) = (P(X-1,Y+1)+P(X,Y+1)+
        sub     ax,bx                   ;   P(X+1,Y+1)+P(X,Y+2)-Seed)/4
        sar     ax,2
        jge     short FlameNotTooLow
        xor     ax,ax
FlameNotTooLow:
        mov     [si-2*XDOTS],ax
        add     si,2
        loop    FlamesTransform

        ; Sets New Random Bottom Flames with Colors from 0 to 127.
        lea     si,[Frame+2*XDOTS*(YDOTS+HIDDEN-2)]
        mov     cx,XDOTS
        xor     dx,dx
RandomFlames:
        call    proc_Random             ; Use Random to Create a New
        test    ax,ax                   ; Flames Bottom Lines
        js      short NotNewColor
        call    proc_Random
        mov     dl,al
        and     dl,7Fh
NotNewColor:
        mov     [si],dx
        mov     [si+2*XDOTS],dx
        add     si,2
        loop    RandomFlames
        mov     ah,1                    ; or Any Key Pressed?
        int     16h                     
        je      short CheckTimer
        mov     ah,0
        int     16h
        mov     bp,TIMEOUT 
        jmp     short DecreaseFlames
CheckTimer:
        cmp     bp,TIMEOUT              ; Timeout?
        jb      FlamesLoop
DecreaseFlames:
        inc     bx                      ; Increase Flames Seed
        cmp     bx,FLBLANK              ; (makes flames more low)
        jbe     short FlamesLoop
FlamesExit:
        mov     ax,0003h                ; Set Text Mode 80x25x16
        int     10h

        int     20h                     ; Exit to DOS.

proc_start      endp

TIMEOUT   equ  70*50                    ; At Least 50 secs
FLSEED    equ  4                        ; Initial Flames Seed
FLBLANK   equ  100                      ; Ending Flames Seed (Blank Screen)
XDOTS     equ  80                       ; VGA Screen Dimensions
YDOTS     equ  80
HIDDEN    equ  8                        ; Number of Hidden Working Lines

;-----------------------------------------------------------------------------
; WaitVR - Waits Vertical Retrace.
;-----------------------------------------------------------------------------

proc_WaitVR     proc near
        mov     dx,3DAh
WaitStartVR:
        in      al,dx
        test    al,8
        je      short WaitStartVR
WaitEndVR:
        in      al,dx
        test    al,8
        jne     short WaitEndVR
        retn
proc_WaitVR     endp

;-----------------------------------------------------------------------------
; Random - Returns a random number of 16 bits.
;-----------------------------------------------------------------------------

proc_Random     proc near
        mov     ax,[Seed]
        imul    ax,8905h
        inc     ax
        mov     [Seed],ax
        retn
proc_Random     endp

Frame        dw   XDOTS*(YDOTS+HIDDEN) dup(?)  ; Flames Frame Buffer
Seed         dw   1234h                 ; Random Seed

Palette label byte                      ; Flames RGB Palette:
        I=0                             ; black to blue
        rept 8
        db   0,0,2*I
        I=I+1
        endm
        I=0                             ; blue to darkred
        rept 8
        db   2*I,0,16-2*I
        I=I+1
        endm
        I=0                             ; darkred to lightred
        rept 24
        db   16+47*I/24,0,0
        I=I+1
        endm
        I=0                             ; lighred to yellow
        rept 32
        db   63,31*I/16,0
        I=I+1
        endm
        I=0                             ; yellow to white
        rept 24
        db   63,63,21*I/8
        I=I+1
        endm
        rept 160                        ; white
        db   63,63,63
        endm
    
        end
